# Originally made by Tamito KAJIYAMA <19 August 2009>
# modified by mato <25 March 2010>

from freestyle_init import *
from PredicatesU1D import *
from logical_operators import *
from shaders import *
import bpy, os

_HEADER = """\
<?xml version="1.0" encoding="utf-8"?>
<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.0//EN"
"http://www.w3.org/TR/2000/CR-SVG-20001102/DTD/svg-20001102.dtd">
<svg xml:space="default" width="%d" height="%d">
"""
_PATH = """\
<path fill="none" stroke="%s" stroke-width="%d" d="M %s" />
"""
_FOOTER = """\
</svg>
"""

class pyGlobalZInspector(UnaryPredicate1D):
    """Inspect the minimum and maximum Z depth values of all vertices
    in the scene."""
    z_min, z_max = 1e+8, -1e+8
    def __call__(self, inter):
        it = inter.verticesBegin()
        while it.isEnd() == 0:
            z = it.getObject().getProjectedZ()
            if z < pyGlobalZInspector.z_min:
                pyGlobalZInspector.z_min = z
            if z > pyGlobalZInspector.z_max:
                pyGlobalZInspector.z_max = z
            it.increment()
        return 1
 
class pyGlobalZDependingThicknessShader(StrokeShader):
    """Give a variable thickness to a stroke depending on Z depth
    values of the vertices in the stroke.  The thickness at a vertex
    is proportional to the vertex's relative Z depth (0.0 to 1.0) in
    the range of global minimum and maximum Z depth values examined by
    pyGlobalZInspector."""
    def __init__(self, min_thickness, max_thickness):
        StrokeShader.__init__(self)
        self.__min = min_thickness
        self.__max = max_thickness
    def shade(self, stroke):
        global_z_min = pyGlobalZInspector.z_min
        global_z_max = pyGlobalZInspector.z_max
        z_diff = 1.0 / (global_z_max - global_z_min)
        z_min, z_max = 1e+8, -1e+8
        it = stroke.strokeVerticesBegin()
        while it.isEnd() == 0:
            v = it.getObject()
            z = (v.getProjectedZ() - global_z_min) * z_diff
            thickness = (1.0 - z) * self.__max + z * self.__min
            v.attribute().setThickness(thickness/2, thickness/2)
            it.increment()
 
class pyTestBP1D(BinaryPredicate1D):
    """True if both operands have the same shape ID and both of them
    are either external contours or non-external contours (i.e. unless
    one operand is an external contour and the other is not)."""
    def __init__(self):
        BinaryPredicate1D.__init__(self)
        self.__bpred = SameShapeIdBP1D()
        self.__upred = ExternalContourUP1D()
    def __call__(self, inter1, inter2):
        return self.__bpred(inter1, inter2) and \
               self.__upred(inter1) == self.__upred(inter2)

class SVGWriter(StrokeShader):
	def __init__(self, f, w, h, start_frame, current_frame, frame_step, fps):
		StrokeShader.__init__(self)
		self.width, self.height = w, h
		self.file = f
		begin = float(current_frame - start_frame) / frame_step / fps
		dur = 1.0 / fps
#		self.file.write(_HEADER % (current_frame, begin, dur))
		self.file.write(_HEADER % (w, h))
	def close(self):
		self.file.write(_FOOTER)
		self.file.close()
	def shade(self, stroke):
		points = []
		for v in stroke:
			x, y = v.getPoint2D()
			points.append("%.3f,%.3f" % (x, self.height - y))
		points = " ".join(points)
		attr = v.attribute()
		r, g, b = attr.getColorRGB() * 255
		color = "#%02x%02x%02x" % (r, g, b)
		width = attr.getThicknessRL()
		width = width[0] + width[1]
		self.file.write(_PATH % (color, width, points))

scene = bpy.data.scenes[0]
render_data = scene.render
start_frame = scene.start_frame
current_frame = scene.current_frame
frame_step = scene.frame_step
fps = render_data.fps
if not os.path.exists(render_data.output_path):
	os.makedirs(render_data.output_path)
path = os.path.join(render_data.output_path, "frame%04d.svg" % current_frame)
f = open(path, "wt")
w = render_data.resolution_x
h = render_data.resolution_y

upred = QuantitativeInvisibilityUP1D(0)
bpred = pyTestBP1D()
Operators.select(upred)
Operators.select(pyGlobalZInspector())
Operators.bidirectionalChain(ChainPredicateIterator(upred, bpred),
                             NotUP1D(upred))
writer = SVGWriter(f, w, h, start_frame, current_frame, frame_step, fps)
shaders_list = [
	pyGlobalZDependingThicknessShader(.1, 5),
	#ConstantThicknessShader(3),
	#pyDepthDiscontinuityThicknessShader(1, 4),
	ConstantColorShader(0, 0, 0),
	#pyMaterialColorShader(0.5),
	#pyDiffuseColorShader(),
	writer,
]
Operators.create(TrueUP1D(), shaders_list)
writer.close()
